package ru.landgrafhomyak.db.sql_skeleton_0.api.raw

import ru.landgrafhomyak.db.sql_skeleton_0.api.misc.DatabaseType
import ru.landgrafhomyak.db.sql_skeleton_0.api.queries.RawStatement


public interface DialectResolver {
	public fun interface Type<RUNTIME_TYPE> {
		public fun resolveByDialect_databaseType(selector: Selector<RUNTIME_TYPE>)

		public interface Selector<RUNTIME_TYPE> {
			@Suppress("BOUNDS_NOT_ALLOWED_IF_BOUNDED_BY_TYPE_PARAMETER")
			public fun <DATABASE_TYPE, DATABASE_TYPE_UNBOUND> bind(
				key: DialectKey<DATABASE_TYPE_UNBOUND, *, *, *, *, *, *, *, *>, type: DATABASE_TYPE
			) where DATABASE_TYPE_UNBOUND : DatabaseType<*>,
					DATABASE_TYPE : DatabaseType<RUNTIME_TYPE>,
					DATABASE_TYPE : DATABASE_TYPE_UNBOUND
		}
	}

	public fun interface Statement_Void2Void<USER_EXTENSION> {
		public fun resolveByDialect_statement_void2void(selector: Selector<USER_EXTENSION>)

		public interface Selector<USER_EXTENSION> {
			@Suppress("BOUNDS_NOT_ALLOWED_IF_BOUNDED_BY_TYPE_PARAMETER")
			public fun <STATEMENT_CONSTRUCTOR, STATEMENT_CONSTRUCTOR_UNBOUND> bind(
				key: DialectKey<*, STATEMENT_CONSTRUCTOR_UNBOUND, *, *, *, *, *, *, *>, constructor: STATEMENT_CONSTRUCTOR
			) where STATEMENT_CONSTRUCTOR_UNBOUND : RawStatement.Void2Void.Constructor<*>,
					STATEMENT_CONSTRUCTOR : RawStatement.Void2Void.Constructor<USER_EXTENSION>,
					STATEMENT_CONSTRUCTOR : STATEMENT_CONSTRUCTOR_UNBOUND
		}
	}

	public fun interface Statement_Void2Row<USER_EXTENSION> {
		public fun resolveByDialect_statement_void2row(selector: Selector<USER_EXTENSION>)

		public interface Selector<USER_EXTENSION> {
			@Suppress("BOUNDS_NOT_ALLOWED_IF_BOUNDED_BY_TYPE_PARAMETER")
			public fun <STATEMENT_CONSTRUCTOR, STATEMENT_CONSTRUCTOR_UNBOUND> bind(
				key: DialectKey<*, *, STATEMENT_CONSTRUCTOR_UNBOUND, *, *, *, *, *, *>, constructor: STATEMENT_CONSTRUCTOR
			) where STATEMENT_CONSTRUCTOR_UNBOUND : RawStatement.Void2Row.Constructor<*>,
					STATEMENT_CONSTRUCTOR : RawStatement.Void2Row.Constructor<USER_EXTENSION>,
					STATEMENT_CONSTRUCTOR : STATEMENT_CONSTRUCTOR_UNBOUND
		}
	}

	public fun interface Statement_Void2RowOrNull<USER_EXTENSION> {
		public fun resolveByDialect_statement_void2rowOrNull(selector: Selector<USER_EXTENSION>)

		public interface Selector<USER_EXTENSION> {
			@Suppress("BOUNDS_NOT_ALLOWED_IF_BOUNDED_BY_TYPE_PARAMETER")
			public fun <STATEMENT_CONSTRUCTOR, STATEMENT_CONSTRUCTOR_UNBOUND> bind(
				key: DialectKey<*, *, *, STATEMENT_CONSTRUCTOR_UNBOUND, *, *, *, *, *>, constructor: STATEMENT_CONSTRUCTOR
			) where STATEMENT_CONSTRUCTOR_UNBOUND : RawStatement.Void2RowOrNull.Constructor<*>,
					STATEMENT_CONSTRUCTOR : RawStatement.Void2RowOrNull.Constructor<USER_EXTENSION>,
					STATEMENT_CONSTRUCTOR : STATEMENT_CONSTRUCTOR_UNBOUND
		}
	}
	public fun interface Statement_Void2Table<USER_EXTENSION> {
		public fun resolveByDialect_statement_void2table(selector: Selector<USER_EXTENSION>)

		public interface Selector<USER_EXTENSION> {
			@Suppress("BOUNDS_NOT_ALLOWED_IF_BOUNDED_BY_TYPE_PARAMETER")
			public fun <STATEMENT_CONSTRUCTOR, STATEMENT_CONSTRUCTOR_UNBOUND> bind(
				key: DialectKey<*, *, *, *, STATEMENT_CONSTRUCTOR_UNBOUND, *, *, *, *>, constructor: STATEMENT_CONSTRUCTOR
			) where STATEMENT_CONSTRUCTOR_UNBOUND : RawStatement.Void2Table.Constructor<*>,
					STATEMENT_CONSTRUCTOR : RawStatement.Void2Table.Constructor<USER_EXTENSION>,
					STATEMENT_CONSTRUCTOR : STATEMENT_CONSTRUCTOR_UNBOUND
		}
	}

	public fun interface Statement_Params2Void<USER_EXTENSION> {
		public fun resolveByDialect_statement_params2void(selector: Selector<USER_EXTENSION>)

		public interface Selector<USER_EXTENSION> {
			@Suppress("BOUNDS_NOT_ALLOWED_IF_BOUNDED_BY_TYPE_PARAMETER")
			public fun <STATEMENT_CONSTRUCTOR, STATEMENT_CONSTRUCTOR_UNBOUND> bind(
				key: DialectKey<*, *, *, *, *, STATEMENT_CONSTRUCTOR_UNBOUND, *, *, *>, constructor: STATEMENT_CONSTRUCTOR
			) where STATEMENT_CONSTRUCTOR_UNBOUND : RawStatement.Params2Void.Constructor<*>,
					STATEMENT_CONSTRUCTOR : RawStatement.Params2Void.Constructor<USER_EXTENSION>,
					STATEMENT_CONSTRUCTOR : STATEMENT_CONSTRUCTOR_UNBOUND
		}
	}

	public fun interface Statement_Params2Row<USER_EXTENSION> {
		public fun resolveByDialect_statement_params2row(selector: Selector<USER_EXTENSION>)

		public interface Selector<USER_EXTENSION> {
			@Suppress("BOUNDS_NOT_ALLOWED_IF_BOUNDED_BY_TYPE_PARAMETER")
			public fun <STATEMENT_CONSTRUCTOR, STATEMENT_CONSTRUCTOR_UNBOUND> bind(
				key: DialectKey<*, *, *, *, *, *, STATEMENT_CONSTRUCTOR_UNBOUND, *, *>, constructor: STATEMENT_CONSTRUCTOR
			) where STATEMENT_CONSTRUCTOR_UNBOUND : RawStatement.Params2Row.Constructor<*>,
					STATEMENT_CONSTRUCTOR : RawStatement.Params2Row.Constructor<USER_EXTENSION>,
					STATEMENT_CONSTRUCTOR : STATEMENT_CONSTRUCTOR_UNBOUND
		}
	}

	public fun interface Statement_Params2RowOrNull<USER_EXTENSION> {
		public fun resolveByDialect_statement_params2rowOrNull(selector: Selector<USER_EXTENSION>)

		public interface Selector<USER_EXTENSION> {
			@Suppress("BOUNDS_NOT_ALLOWED_IF_BOUNDED_BY_TYPE_PARAMETER")
			public fun <STATEMENT_CONSTRUCTOR, STATEMENT_CONSTRUCTOR_UNBOUND> bind(
				key: DialectKey<*, *, *, *, *, *, *, STATEMENT_CONSTRUCTOR_UNBOUND, *>, constructor: STATEMENT_CONSTRUCTOR
			) where STATEMENT_CONSTRUCTOR_UNBOUND : RawStatement.Params2RowOrNull.Constructor<*>,
					STATEMENT_CONSTRUCTOR : RawStatement.Params2RowOrNull.Constructor<USER_EXTENSION>,
					STATEMENT_CONSTRUCTOR : STATEMENT_CONSTRUCTOR_UNBOUND
		}
	}
	public fun interface Statement_Params2Table<USER_EXTENSION> {
		public fun resolveByDialect_statement_params2table(selector: Selector<USER_EXTENSION>)

		public interface Selector<USER_EXTENSION> {
			@Suppress("BOUNDS_NOT_ALLOWED_IF_BOUNDED_BY_TYPE_PARAMETER")
			public fun <STATEMENT_CONSTRUCTOR, STATEMENT_CONSTRUCTOR_UNBOUND> bind(
				key: DialectKey<*, *, *, *, *, *, *, *, STATEMENT_CONSTRUCTOR_UNBOUND>, constructor: STATEMENT_CONSTRUCTOR
			) where STATEMENT_CONSTRUCTOR_UNBOUND : RawStatement.Params2Table.Constructor<*>,
					STATEMENT_CONSTRUCTOR : RawStatement.Params2Table.Constructor<USER_EXTENSION>,
					STATEMENT_CONSTRUCTOR : STATEMENT_CONSTRUCTOR_UNBOUND
		}
	}
}